@@ -5,21 +5,28 @@ from typing import Dict, List, Tuple, Any
 
 import ray
 from ray.rllib.evaluation.worker_set import WorkerSet
-from ray.rllib.execution.common import \
-    AGENT_STEPS_TRAINED_COUNTER, APPLY_GRADS_TIMER, COMPUTE_GRADS_TIMER, \
-    LAST_TARGET_UPDATE_TS, LEARN_ON_BATCH_TIMER, \
-    LOAD_BATCH_TIMER, NUM_TARGET_UPDATES, STEPS_SAMPLED_COUNTER, \
-    STEPS_TRAINED_COUNTER, STEPS_TRAINED_THIS_ITER_COUNTER, \
-    WORKER_UPDATE_TIMER, _check_sample_batch_type, \
-    _get_global_vars, _get_shared_metrics
+from ray.rllib.execution.common import (
+    AGENT_STEPS_TRAINED_COUNTER,
+    APPLY_GRADS_TIMER,
+    COMPUTE_GRADS_TIMER,
+    LAST_TARGET_UPDATE_TS,
+    LEARN_ON_BATCH_TIMER,
+    LOAD_BATCH_TIMER,
+    NUM_TARGET_UPDATES,
+    STEPS_SAMPLED_COUNTER,
+    STEPS_TRAINED_COUNTER,
+    STEPS_TRAINED_THIS_ITER_COUNTER,
+    WORKER_UPDATE_TIMER,
+    _check_sample_batch_type,
+    _get_global_vars,
+    _get_shared_metrics,
+)
 from ray.rllib.policy.sample_batch import DEFAULT_POLICY_ID, MultiAgentBatch
 from ray.rllib.utils.annotations import ExperimentalAPI
 from ray.rllib.utils.deprecation import DEPRECATED_VALUE, deprecation_warning
 from ray.rllib.utils.framework import try_import_tf
-from ray.rllib.utils.metrics import NUM_ENV_STEPS_TRAINED, \
-    NUM_AGENT_STEPS_TRAINED
-from ray.rllib.utils.metrics.learner_info import LearnerInfoBuilder, \
-    LEARNER_INFO
+from ray.rllib.utils.metrics import NUM_ENV_STEPS_TRAINED, NUM_AGENT_STEPS_TRAINED
+from ray.rllib.utils.metrics.learner_info import LearnerInfoBuilder, LEARNER_INFO
 from ray.rllib.utils.sgd import do_minibatch_sgd
 from ray.rllib.utils.typing import PolicyID, SampleBatchType, ModelGradients
 
@@ -33,7 +40,6 @@ def train_one_step(trainer, train_batch) -> Dict:
     config = trainer.config
     workers = trainer.workers
     local_worker = workers.local_worker()
-    policies = local_worker.policies_to_train
     num_sgd_iter = config.get("num_sgd_iter", 1)
     sgd_minibatch_size = config.get("sgd_minibatch_size", 0)
 
@@ -44,9 +50,15 @@ def train_one_step(trainer, train_batch) -> Dict:
         if num_sgd_iter > 1 or sgd_minibatch_size > 0:
             info = do_minibatch_sgd(
                 train_batch,
-                {pid: local_worker.get_policy(pid)
-                 for pid in policies}, local_worker, num_sgd_iter,
-                sgd_minibatch_size, [])
+                {
+                    pid: local_worker.get_policy(pid)
+                    for pid in local_worker.get_policies_to_train(train_batch)
+                },
+                local_worker,
+                num_sgd_iter,
+                sgd_minibatch_size,
+                [],
+            )
         # Single update step using train batch.
         else:
             info = local_worker.learn_on_batch(train_batch)
@@ -63,10 +75,8 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
     config = trainer.config
     workers = trainer.workers
     local_worker = workers.local_worker()
-    policies = local_worker.policies_to_train
     num_sgd_iter = config.get("sgd_num_iter", 1)
-    sgd_minibatch_size = config.get("sgd_minibatch_size",
-                                    config["train_batch_size"])
+    sgd_minibatch_size = config.get("sgd_minibatch_size", config["train_batch_size"])
 
     # Determine the number of devices (GPUs or 1 CPU) we use.
     num_devices = int(math.ceil(config["num_gpus"] or 1))
@@ -88,7 +98,7 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
         num_loaded_samples = {}
         for policy_id, batch in train_batch.policy_batches.items():
             # Not a policy-to-train.
-            if policy_id not in policies:
+            if not local_worker.is_policy_to_train(policy_id, train_batch):
                 continue
 
             # Decompress SampleBatch, in case some columns are compressed.
@@ -98,8 +108,8 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
             # (idx=0). Policies only have >1 buffers, if we are training
             # asynchronously.
             num_loaded_samples[policy_id] = local_worker.policy_map[
-                policy_id].load_batch_into_buffer(
-                    batch, buffer_index=0)
+                policy_id
+            ].load_batch_into_buffer(batch, buffer_index=0)
 
     # Execute minibatch SGD on loaded data.
     learn_timer = trainer._timers[LEARN_ON_BATCH_TIMER]
@@ -113,9 +123,7 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
 
         for policy_id, samples_per_device in num_loaded_samples.items():
             policy = local_worker.policy_map[policy_id]
-            num_batches = max(
-                1,
-                int(samples_per_device) // int(per_device_batch_size))
+            num_batches = max(1, int(samples_per_device) // int(per_device_batch_size))
             logger.debug("== sgd epochs for {} ==".format(policy_id))
             for _ in range(num_sgd_iter):
                 permutation = np.random.permutation(num_batches)
@@ -124,11 +132,10 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
                     # Note: For minibatch SGD, the data is an offset into
                     # the pre-loaded entire train batch.
                     results = policy.learn_on_loaded_batch(
-                        permutation[batch_index] * per_device_batch_size,
-                        buffer_index=0)
+                        permutation[batch_index] * per_device_batch_size, buffer_index=0
+                    )
 
-                    learner_info_builder.add_learn_on_batch_results(
-                        results, policy_id)
+                    learner_info_builder.add_learn_on_batch_results(results, policy_id)
 
         # Tower reduce and finalize results.
         learner_info = learner_info_builder.finalize()
@@ -143,7 +150,11 @@ def multi_gpu_train_one_step(trainer, train_batch) -> Dict:
     # workers.
     if workers.remote_workers():
         with trainer._timers[WORKER_UPDATE_TIMER]:
-            weights = ray.put(workers.local_worker().get_weights(policies))
+            weights = ray.put(
+                local_worker.get_weights(
+                    local_worker.get_policies_to_train(train_batch)
+                )
+            )
             for e in workers.remote_workers():
                 e.set_weights.remote(weights)
 
@@ -166,55 +177,60 @@ class TrainOneStep:
     local iterator context.
     """
 
-    def __init__(self,
-                 workers: WorkerSet,
-                 policies: List[PolicyID] = frozenset([]),
-                 num_sgd_iter: int = 1,
-                 sgd_minibatch_size: int = 0):
+    def __init__(
+        self,
+        workers: WorkerSet,
+        policies: List[PolicyID] = frozenset([]),
+        num_sgd_iter: int = 1,
+        sgd_minibatch_size: int = 0,
+    ):
         self.workers = workers
         self.local_worker = workers.local_worker()
         self.policies = policies
         self.num_sgd_iter = num_sgd_iter
         self.sgd_minibatch_size = sgd_minibatch_size
 
-    def __call__(self,
-                 batch: SampleBatchType) -> (SampleBatchType, List[dict]):
+    def __call__(self, batch: SampleBatchType) -> (SampleBatchType, List[dict]):
         _check_sample_batch_type(batch)
         metrics = _get_shared_metrics()
         learn_timer = metrics.timers[LEARN_ON_BATCH_TIMER]
+        lw = self.local_worker
         with learn_timer:
             # Subsample minibatches (size=`sgd_minibatch_size`) from the
             # train batch and loop through train batch `num_sgd_iter` times.
             if self.num_sgd_iter > 1 or self.sgd_minibatch_size > 0:
-                lw = self.workers.local_worker()
                 learner_info = do_minibatch_sgd(
-                    batch, {
+                    batch,
+                    {
                         pid: lw.get_policy(pid)
-                        for pid in self.policies
-                        or self.local_worker.policies_to_train
-                    }, lw, self.num_sgd_iter, self.sgd_minibatch_size, [])
+                        for pid in self.policies or lw.get_policies_to_train(batch)
+                    },
+                    lw,
+                    self.num_sgd_iter,
+                    self.sgd_minibatch_size,
+                    [],
+                )
             # Single update step using train batch.
             else:
-                learner_info = \
-                    self.workers.local_worker().learn_on_batch(batch)
+                learner_info = lw.learn_on_batch(batch)
 
             metrics.info[LEARNER_INFO] = learner_info
             learn_timer.push_units_processed(batch.count)
         metrics.counters[STEPS_TRAINED_COUNTER] += batch.count
         metrics.counters[STEPS_TRAINED_THIS_ITER_COUNTER] = batch.count
         if isinstance(batch, MultiAgentBatch):
-            metrics.counters[
-                AGENT_STEPS_TRAINED_COUNTER] += batch.agent_steps()
+            metrics.counters[AGENT_STEPS_TRAINED_COUNTER] += batch.agent_steps()
         # Update weights - after learning on the local worker - on all remote
         # workers.
         if self.workers.remote_workers():
             with metrics.timers[WORKER_UPDATE_TIMER]:
-                weights = ray.put(self.workers.local_worker().get_weights(
-                    self.policies or self.local_worker.policies_to_train))
+                weights = ray.put(
+                    lw.get_weights(self.policies or lw.get_policies_to_train(batch))
+                )
                 for e in self.workers.remote_workers():
                     e.set_weights.remote(weights, _get_global_vars())
         # Also update global vars of the local worker.
-        self.workers.local_worker().set_global_vars(_get_global_vars())
+        lw.set_global_vars(_get_global_vars())
         return batch, learner_info
 
 
@@ -235,22 +251,22 @@ class MultiGPUTrainOneStep:
     """
 
     def __init__(
-            self,
-            *,
-            workers: WorkerSet,
-            sgd_minibatch_size: int,
-            num_sgd_iter: int,
-            num_gpus: int,
-            _fake_gpus: bool = False,
-            # Deprecated args.
-            shuffle_sequences=DEPRECATED_VALUE,
-            framework=DEPRECATED_VALUE):
-        if framework != DEPRECATED_VALUE or \
-                shuffle_sequences != DEPRECATED_VALUE:
+        self,
+        *,
+        workers: WorkerSet,
+        sgd_minibatch_size: int,
+        num_sgd_iter: int,
+        num_gpus: int,
+        _fake_gpus: bool = False,
+        # Deprecated args.
+        shuffle_sequences=DEPRECATED_VALUE,
+        framework=DEPRECATED_VALUE
+    ):
+        if framework != DEPRECATED_VALUE or shuffle_sequences != DEPRECATED_VALUE:
             deprecation_warning(
-                old="MultiGPUTrainOneStep(framework=..., "
-                "shuffle_sequences=...)",
-                error=False)
+                old="MultiGPUTrainOneStep(framework=..., shuffle_sequences=...)",
+                error=False,
+            )
 
         self.workers = workers
         self.local_worker = workers.local_worker()
@@ -276,8 +292,7 @@ class MultiGPUTrainOneStep:
         assert self.batch_size % len(self.devices) == 0
         assert self.batch_size >= len(self.devices), "Batch size too small!"
 
-    def __call__(self,
-                 samples: SampleBatchType) -> (SampleBatchType, List[dict]):
+    def __call__(self, samples: SampleBatchType) -> (SampleBatchType, List[dict]):
         _check_sample_batch_type(samples)
 
         # Handle everything as if multi agent.
@@ -291,7 +306,7 @@ class MultiGPUTrainOneStep:
             num_loaded_samples = {}
             for policy_id, batch in samples.policy_batches.items():
                 # Not a policy-to-train.
-                if policy_id not in self.local_worker.policies_to_train:
+                if not self.local_worker.is_policy_to_train(policy_id, samples):
                     continue
 
                 # Decompress SampleBatch, in case some columns are compressed.
@@ -301,8 +316,8 @@ class MultiGPUTrainOneStep:
                 # (idx=0). Policies only have >1 buffers, if we are training
                 # asynchronously.
                 num_loaded_samples[policy_id] = self.local_worker.policy_map[
-                    policy_id].load_batch_into_buffer(
-                        batch, buffer_index=0)
+                    policy_id
+                ].load_batch_into_buffer(batch, buffer_index=0)
 
         # Execute minibatch SGD on loaded data.
         with learn_timer:
@@ -311,14 +326,13 @@ class MultiGPUTrainOneStep:
             # This makes sure results dicts always have the same structure
             # no matter the setup (multi-GPU, multi-agent, minibatch SGD,
             # tf vs torch).
-            learner_info_builder = LearnerInfoBuilder(
-                num_devices=len(self.devices))
+            learner_info_builder = LearnerInfoBuilder(num_devices=len(self.devices))
 
             for policy_id, samples_per_device in num_loaded_samples.items():
                 policy = self.local_worker.policy_map[policy_id]
                 num_batches = max(
-                    1,
-                    int(samples_per_device) // int(self.per_device_batch_size))
+                    1, int(samples_per_device) // int(self.per_device_batch_size)
+                )
                 logger.debug("== sgd epochs for {} ==".format(policy_id))
                 for _ in range(self.num_sgd_iter):
                     permutation = np.random.permutation(num_batches)
@@ -327,12 +341,13 @@ class MultiGPUTrainOneStep:
                         # Note: For minibatch SGD, the data is an offset into
                         # the pre-loaded entire train batch.
                         results = policy.learn_on_loaded_batch(
-                            permutation[batch_index] *
-                            self.per_device_batch_size,
-                            buffer_index=0)
+                            permutation[batch_index] * self.per_device_batch_size,
+                            buffer_index=0,
+                        )
 
                         learner_info_builder.add_learn_on_batch_results(
-                            results, policy_id)
+                            results, policy_id
+                        )
 
             # Tower reduce and finalize results.
             learner_info = learner_info_builder.finalize()
@@ -347,8 +362,11 @@ class MultiGPUTrainOneStep:
 
         if self.workers.remote_workers():
             with metrics.timers[WORKER_UPDATE_TIMER]:
-                weights = ray.put(self.workers.local_worker().get_weights(
-                    self.local_worker.policies_to_train))
+                weights = ray.put(
+                    self.workers.local_worker().get_weights(
+                        self.local_worker.get_policies_to_train()
+                    )
+                )
                 for e in self.workers.remote_workers():
                     e.set_weights.remote(weights, _get_global_vars())
 
@@ -381,13 +399,11 @@ class ComputeGradients:
         _check_sample_batch_type(samples)
         metrics = _get_shared_metrics()
         with metrics.timers[COMPUTE_GRADS_TIMER]:
-            grad, info = self.workers.local_worker().compute_gradients(samples)
-        # RolloutWorker.compute_gradients returns pure single agent stats
-        # in a non-multi agent setup.
-        if isinstance(samples, MultiAgentBatch):
-            metrics.info[LEARNER_INFO] = info
-        else:
-            metrics.info[LEARNER_INFO] = {DEFAULT_POLICY_ID: info}
+            grad, info = self.workers.local_worker().compute_gradients(
+                samples, single_agent=True
+            )
+        # RolloutWorker.compute_gradients returned single-agent stats.
+        metrics.info[LEARNER_INFO] = {DEFAULT_POLICY_ID: info}
         return grad, samples.count
 
 
@@ -404,10 +420,9 @@ class ApplyGradients:
     Updates the STEPS_TRAINED_COUNTER counter in the local iterator context.
     """
 
-    def __init__(self,
-                 workers,
-                 policies: List[PolicyID] = frozenset([]),
-                 update_all=True):
+    def __init__(
+        self, workers, policies: List[PolicyID] = frozenset([]), update_all=True
+    ):
         """Creates an ApplyGradients instance.
 
         Args:
@@ -424,8 +439,8 @@ class ApplyGradients:
     def __call__(self, item: Tuple[ModelGradients, int]) -> None:
         if not isinstance(item, tuple) or len(item) != 2:
             raise ValueError(
-                "Input must be a tuple of (grad_dict, count), got {}".format(
-                    item))
+                "Input must be a tuple of (grad_dict, count), got {}".format(item)
+            )
         gradients, count = item
         metrics = _get_shared_metrics()
         metrics.counters[STEPS_TRAINED_COUNTER] += count
@@ -433,17 +448,20 @@ class ApplyGradients:
 
         apply_timer = metrics.timers[APPLY_GRADS_TIMER]
         with apply_timer:
-            self.workers.local_worker().apply_gradients(gradients)
+            self.local_worker.apply_gradients(gradients)
             apply_timer.push_units_processed(count)
 
         # Also update global vars of the local worker.
-        self.workers.local_worker().set_global_vars(_get_global_vars())
+        self.local_worker.set_global_vars(_get_global_vars())
 
         if self.update_all:
             if self.workers.remote_workers():
                 with metrics.timers[WORKER_UPDATE_TIMER]:
-                    weights = ray.put(self.workers.local_worker().get_weights(
-                        self.policies or self.local_worker.policies_to_train))
+                    weights = ray.put(
+                        self.local_worker.get_weights(
+                            self.policies or self.local_worker.get_policies_to_train()
+                        )
+                    )
                     for e in self.workers.remote_workers():
                         e.set_weights.remote(weights, _get_global_vars())
         else:
@@ -451,12 +469,13 @@ class ApplyGradients:
                 raise ValueError(
                     "Could not find actor to update. When "
                     "update_all=False, `current_actor` must be set "
-                    "in the iterator context.")
+                    "in the iterator context."
+                )
             with metrics.timers[WORKER_UPDATE_TIMER]:
-                weights = self.workers.local_worker().get_weights(
-                    self.policies or self.local_worker.policies_to_train)
-                metrics.current_actor.set_weights.remote(
-                    weights, _get_global_vars())
+                weights = self.local_worker.get_weights(
+                    self.policies or self.local_worker.get_policies_to_train()
+                )
+                metrics.current_actor.set_weights.remote(weights, _get_global_vars())
 
 
 class AverageGradients:
@@ -472,8 +491,9 @@ class AverageGradients:
         {"var_0": ..., ...}, 1600  # averaged grads, summed batch count
     """
 
-    def __call__(self, gradients: List[Tuple[ModelGradients, int]]
-                 ) -> Tuple[ModelGradients, int]:
+    def __call__(
+        self, gradients: List[Tuple[ModelGradients, int]]
+    ) -> Tuple[ModelGradients, int]:
         acc = None
         sum_count = 0
         for grad, count in gradients:
@@ -482,8 +502,10 @@ class AverageGradients:
             else:
                 acc = [a + b for a, b in zip(acc, grad)]
             sum_count += count
-        logger.info("Computing average of {} microbatch gradients "
-                    "({} samples total)".format(len(gradients), sum_count))
+        logger.info(
+            "Computing average of {} microbatch gradients "
+            "({} samples total)".format(len(gradients), sum_count)
+        )
         return acc, sum_count
 
 
@@ -505,11 +527,13 @@ class UpdateTargetNetwork:
     track when we should update the target next.
     """
 
-    def __init__(self,
-                 workers: WorkerSet,
-                 target_update_freq: int,
-                 by_steps_trained: bool = False,
-                 policies: List[PolicyID] = frozenset([])):
+    def __init__(
+        self,
+        workers: WorkerSet,
+        target_update_freq: int,
+        by_steps_trained: bool = False,
+        policies: List[PolicyID] = frozenset([]),
+    ):
         self.workers = workers
         self.local_worker = workers.local_worker()
         self.target_update_freq = target_update_freq
@@ -523,9 +547,10 @@ class UpdateTargetNetwork:
         metrics = _get_shared_metrics()
         cur_ts = metrics.counters[self.metric]
         last_update = metrics.counters[LAST_TARGET_UPDATE_TS]
-        if cur_ts - last_update > self.target_update_freq:
-            to_update = self.policies or self.local_worker.policies_to_train
-            self.workers.local_worker().foreach_trainable_policy(
-                lambda p, p_id: p_id in to_update and p.update_target())
+        if cur_ts - last_update >= self.target_update_freq:
+            to_update = self.policies or self.local_worker.get_policies_to_train()
+            self.workers.local_worker().foreach_policy_to_train(
+                lambda p, pid: pid in to_update and p.update_target()
+            )
             metrics.counters[NUM_TARGET_UPDATES] += 1
             metrics.counters[LAST_TARGET_UPDATE_TS] = cur_ts
